@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT7 ****************************************
 
-Copyright (c) 2015-2018, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -63,6 +63,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "cs_csm_log.h"
 #include "iba/public/imath.h"
 #include "if3.h"
+#include "pm_l.h"
 
 
 #ifndef __VXWORKS__
@@ -421,7 +422,7 @@ void sm_init_plt_table(void){
                 sa_dynamicPlt[6], sa_dynamicPlt[7], sa_dynamicPlt[8], sa_dynamicPlt[9]);
         vs_log_output_message(msgbuf, FALSE);
     } else {
-        sprintf(msgbuf, "SM: Dynamic packet lifetime is OFF, using saPacketLifetime contant %d", (unsigned int)sm_config.sa_packet_lifetime_n2);
+        sprintf(msgbuf, "SM: Dynamic packet lifetime is OFF, using saPacketLifetime constant %d", (unsigned int)sm_config.sa_packet_lifetime_n2);
         vs_log_output_message(msgbuf, FALSE);
     }
 	return;
@@ -2416,6 +2417,8 @@ smProcessReconfigureRequest(void){
 					}
 					updatedVirtualFabrics = newVirtualFabrics;
 
+					pm_update_dyn_config(&new_xml_config->fm_instance[sm_instance]->pm_config);
+
 					/* Update our consistency checksums */
 					sm_dbsync_checksums(savedVfConsistencyChecksum,
 										new_xml_config->fm_instance[sm_instance]->sm_config.consistency_checksum,
@@ -2457,21 +2460,6 @@ smProcessReconfigureRequest(void){
 						}
 						// After a reconfiguration, force a resweep
 						sm_trigger_sweep(SM_SWEEP_REASON_RECONFIG);
-					} else if (sm_state == SM_STATE_STANDBY) {
-						SmRecKey_t reckey = sm_smInfo.PortGUID;  /* our guid */
-						SmRecp     smrecp;
-						Status_t   status;
-
-						if ((status = vs_lock(&smRecords.smLock)) != VSTATUS_OK) {
-							IB_LOG_ERRORRC("Can't lock SM Record table, rc:", status);
-						} else {
-        					/* fetch our current dbsync settings */
-        					if ((smrecp = (SmRecp)cs_hashtable_search(smRecords.smMap, &reckey)) != NULL) {
-            					/* Set our version to current value */
-            					smrecp->dbsync.version = SM_DBSYNC_VERSION;
-							}
-							vs_unlock(&smRecords.smLock);
-						}
 					}
 					IB_LOG_INFINI_INFO0("SM: Reconfiguration completed successfully");
 				}
@@ -2483,7 +2471,7 @@ smProcessReconfigureRequest(void){
 			}
 
 		} else {
-			IB_LOG_ERROR0("SM: Error processing reconfigure request; reRenderVirtaulFabricsConfig failed");
+			IB_LOG_ERROR0("SM: Error processing reconfigure request; reRenderVirtualFabricsConfig failed");
 		}
 
 	}
@@ -2491,6 +2479,24 @@ smProcessReconfigureRequest(void){
 	//Optimize: release temp xml memory on sm cleanup, not every time we process
 	//reconfigure request
 
+
+	if (sm_state == SM_STATE_STANDBY) {
+		SmRecKey_t reckey = sm_smInfo.PortGUID;  /* our guid */
+		SmRecp     smrecp;
+		Status_t   status;
+
+		if ((status = vs_lock(&smRecords.smLock)) != VSTATUS_OK) {
+			IB_LOG_ERRORRC("Can't lock SM Record table, rc:", status);
+		} else {
+       		/* fetch our current dbsync settings */
+       		if ((smrecp = (SmRecp)cs_hashtable_search(smRecords.smMap, &reckey)) != NULL) {
+       			/* Set our version to current value */
+       			smrecp->dbsync.version = SM_DBSYNC_VERSION;
+			}
+			vs_unlock(&smRecords.smLock);
+		}
+	}
+
 	// Allow sweeps to run
 	(void)vs_unlock(&new_topology_lock);
 
