@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT7 ****************************************
 
-Copyright (c) 2015-2018, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -75,7 +75,6 @@ Lock_t sm_mcSpanningTreeRootGuidLock;
 /* Number of classes that actually exist */
 uint16_t numMcGroupClasses = 0;
 
-
 McGroupClass_t mcGroupClasses[MAX_SUPPORTED_MCAST_GRP_CLASSES];
 McGroupClass_t defaultMcGroupClass = {
 	.maximumLids = 0,
@@ -84,7 +83,9 @@ McGroupClass_t defaultMcGroupClass = {
 	.value.Raw = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
-void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST);
+extern VirtualFabrics_t *updatedVirtualFabrics;
+
+void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST, bitset_t *allmlids);
 
 static __inline__
 void Dump_SpanningTrees(uint8_t MTU, uint8_t RATE) {
@@ -154,14 +155,11 @@ Status_t sm_ideal_spanning_tree(McSpanningTree_t *mcST, int filter_mtu_rate, int
 			if (!sm_valid_port(portp)) {
 				continue;
 			}
-			if (filter_mtu_rate) {
-				if ( portp->state < IB_PORT_ACTIVE) {
-					continue;
-				}
-			} else {
-				if ( portp->state <= IB_PORT_DOWN) {
-					continue;
-				}
+			if (portp->state < IB_PORT_INIT) {
+				continue;
+			}
+			if (filter_mtu_rate && portp->state < IB_PORT_ACTIVE) {
+				continue;
 			}
 
 			if (portp->nodeno == (int32_t)-1 || portp->portno == (int32_t)-1) {	// switch to switch link broken
@@ -391,7 +389,7 @@ sm_spanning_tree(int32_t mtu, int32_t rate, int *complete) {
 	for (i = 0; (tp->num_sws > 0) && (i < tp->num_sws-1); i++) {
 		if ((nodep = sm_find_node(sm_topop, mcNodes[i].index)) == NULL) continue;
 		for_all_physical_ports(nodep, portp) {
-			if (!sm_valid_port(portp) || portp->state < IB_PORT_ACTIVE) {	// not ACTIVE, skip
+			if (!sm_valid_port(portp) || portp->state < IB_PORT_ACTIVE) {
 				continue;
 			}
 
@@ -553,7 +551,8 @@ findFirstPortInMftBlock(STL_PORTMASK *mftBlock, int blockPos) {
 #define	Mft_Position(X)			((X)/STL_PORT_MASK_WIDTH)
 #define	Mft_PortmaskBit(X)		((uint64)1 << (X)%STL_PORT_MASK_WIDTH)
 
-static Status_t sm_createMCastGrp(int vf, uint64_t* mgid, uint16_t pkey, uint8_t mtu, uint8_t rate, uint8_t sl, uint32_t qkey, uint32_t fl, uint8_t tc) {
+static Status_t sm_createMCastGrp(int vf, uint64_t* mgid, uint16_t pkey, uint8_t mtu, uint8_t rate,
+								  uint8_t sl, uint32_t qkey, uint32_t fl, uint8_t tc, STL_LID preset_mlid) {
 
 	Status_t	status;
 	McGroup_t	*mcGroup;
@@ -564,10 +563,11 @@ static Status_t sm_createMCastGrp(int vf, uint64_t* mgid, uint16_t pkey, uint8_t
 
 	mGid.AsReg64s.H = mgid[0];
 	mGid.AsReg64s.L = mgid[1];
-	if ((status = sm_multicast_assign_lid(mGid, pkey, mtu, rate, &mLid)) != VSTATUS_OK) {
+	if ((status = sm_multicast_assign_lid(mGid, pkey, mtu, rate, preset_mlid, &mLid)) != VSTATUS_OK) {
 		sysPrintf("Failed to allocate LID for Multicast group!\n");
 		return status;
 	}
+
 	McGroup_Create(mcGroup); //create register and add it to structure
 
 	mcGroup->mGid.AsReg64s.H = mgid[0];
@@ -583,7 +583,7 @@ static Status_t sm_createMCastGrp(int vf, uint64_t* mgid, uint16_t pkey, uint8_t
 	mcGroup->hopLimit = 0xFF;
 	mcGroup->scope = IB_LINK_LOCAL_SCOPE;
 	mcGroup->members_full = 1;
-	bitset_set(&mcGroup->vfMembers, vf);
+	bitset_set(&mcGroup->new_vfMembers, vf);
 	//also add dummy McMember
 	McMember_Create(mcGroup, mcMember);
 	mcMember->slid = 0;
@@ -621,8 +621,7 @@ static Status_t sm_createMCastGrp(int vf, uint64_t* mgid, uint16_t pkey, uint8_t
 
 Status_t sm_update_mc_groups(VirtualFabrics_t *newVF, VirtualFabrics_t *oldVF)
 {
-	McGroup_t *mcGroup, *tmpGroup;
-	McMember_t *mcMember;
+	McGroup_t *mcGroup;
 	uint32_t vf;
 	VFDg_t *dg_ref;
 	VFAppMgid_t *mgid_ref;
@@ -638,66 +637,23 @@ Status_t sm_update_mc_groups(VirtualFabrics_t *newVF, VirtualFabrics_t *oldVF)
 		return status;
 	}
 
-	for (vf=0; vf < oldVF->number_of_vfs_all; vf++) {
-		// VF transitions from active to standby
-		if (oldVF->v_fabric_all[vf].removed) {
-			IB_LOG_INFINI_INFO_FMT(__func__, "VF %s change from ACT to STB or REMOVED", oldVF->v_fabric_all[vf].name );
-			//delete if necessary mcgroups
-			mcGroup = sm_McGroups;
-			while (mcGroup) {
-				if (bitset_test(&mcGroup->vfMembers, vf)) {
-					//find members in VF
-					mcMember = mcGroup->mcMembers;
-					for_all_multicast_members(mcGroup, mcMember) {
-						uint64_t mcMemberGuid;
-						Port_t *portp;
-						mcMemberGuid = mcGroup->mcMembers->record.RID.PortGID.Type.Global.InterfaceID;
-						if (mcMemberGuid != 0) {
-							if ((portp = sm_find_port_guid(&old_topology, mcGroup->mcMembers->portGuid)) != NULL) {
-								if (bitset_test(&portp->portData->vfMember,vf)) {
-									IB_LOG_INFINI_INFO_FMT(__func__, "VF %s  port_guid "FMT_U64" moved to STB\n",
-										oldVF->v_fabric_all[vf].name, mcGroup->mcMembers->portGuid);
-									//mark node that is not a member
-									if (portp->portData->portInfo.PortStates.s.PortState == IB_PORT_ACTIVE) {
-									// Indicate that reregistration is pending; will persist across
-									// sweeps until it succeeds or is no longer required (e.g. state change)
-										portp->portData->reregisterPending = 1;
-									}
-									bitset_clear(&portp->portData->vfMember,vf);
-								}
-							}
-						}
-					}
-					//reset bit for that VF
-					bitset_clear(&mcGroup->vfMembers, vf);
-				}
-				// if there is no other VF matching, check it has no members and then delete it
-				if (bitset_nset(&mcGroup->vfMembers)== 0) {
-					while (mcGroup->mcMembers) {
-						// MUST copy head pointer into temp
-						// Passing mcGroup->members directly to the delete macro will corrupt the list
-						mcMember = mcGroup->mcMembers;
-						McMember_Delete(mcGroup, mcMember);
-					}
-					if (mcGroup->mcMembers == NULL) { // Group is empty, delete it
-						IB_LOG_INFINI_INFO_FMT(__func__, "VF %s. Group is empty, deleting MGID: "FMT_GID, oldVF->v_fabric_all[vf].name, mcGroup->mGid.Type.Global.SubnetPrefix, mcGroup->mGid.Type.Global.InterfaceID);
-						tmpGroup = mcGroup->next;
-						McGroup_Delete(mcGroup);
-						mcGroup = tmpGroup;
-					}
-				}
-				else
-					mcGroup = mcGroup->next;
-			} // while mcGroup ends
-		} // if end
-	} // for all old-vfs end
+	// convert VF Indexes from old to new
+	for_all_multicast_groups(mcGroup) {
+		int vfx;
+		bitset_clear_all(&mcGroup->new_vfMembers);
+		for (vfx = 0; (vfx = bitset_find_next_one(&mcGroup->vfMembers, vfx)) != -1; vfx++) {
+			if (oldVF->v_fabric_all[vfx].reconf_idx.new_index < MAX_VFABRICS) {
+				bitset_set(&mcGroup->new_vfMembers, oldVF->v_fabric_all[vfx].reconf_idx.new_index);
+			}
+		}
+	}
 
 	for (vf=0; vf < newVF->number_of_vfs_all; vf++) {
 		//order in v_fabric_all remains same when changing status (Active/Standby)
 		// from old to new.
 		// swap from standby to active
 		uint32_t qos_idx = newVF->v_fabric_all[vf].qos_index;
-		if (newVF->v_fabric_all[vf].added) {
+		if (newVF->v_fabric_all[vf].reconf_idx.old_index == (uint32_t)(-1)) {
 			// add if necessary mcgroups
 			IB_LOG_INFINI_INFO_FMT(__func__,
 					"VF %s change from STB to ACT or CREATE",newVF->v_fabric_all[vf].name );
@@ -717,7 +673,8 @@ Status_t sm_update_mc_groups(VirtualFabrics_t *newVF, VirtualFabrics_t *oldVF)
 								sl = newVF->qos_all[qos_idx].mcast_sl;
 							else sl = newVF->qos_all[qos_idx].base_sl;
 							status = sm_createMCastGrp(vf, mgid_ref->mgid, dg_ref->def_mc_pkey, dg_ref->def_mc_mtu_int,
-								dg_ref->def_mc_rate_int, sl ,dg_ref->def_mc_qkey, dg_ref->def_mc_fl, dg_ref->def_mc_tc);
+													   dg_ref->def_mc_rate_int, sl ,dg_ref->def_mc_qkey, 
+													   dg_ref->def_mc_fl, dg_ref->def_mc_tc, dg_ref->def_mc_mlid);
 							if (status == VSTATUS_OK) {
 								IB_LOG_INFINI_INFO_FMT(__func__, "Creating multicast MGID: "FMT_GID, mgid_ref->mgid[0], mgid_ref->mgid[1]);
 							}
@@ -725,7 +682,7 @@ Status_t sm_update_mc_groups(VirtualFabrics_t *newVF, VirtualFabrics_t *oldVF)
 						}
 						else {
 							IB_LOG_INFINI_INFO_FMT(__func__," Multicast group exists MGID: "FMT_GID, mgid_ref->mgid[0], mgid_ref->mgid[1]);
-							bitset_set(&mcGroup->vfMembers, vf); // add current VF to the mgid
+							bitset_set(&mcGroup->new_vfMembers, vf); // add current VF to the mgid
 						}
 					}
 				}
@@ -756,6 +713,10 @@ void sm_check_mc_groups_realizable(void)
 	topo = &sm_newTopology;
 
 	for_all_multicast_groups(mcGroup) {
+		if (updatedVirtualFabrics && !bitset_nset(&mcGroup->new_vfMembers)) {
+			// If reconfig sweep, and new vfs is empty do not add to MFT
+			continue;
+		}
 		tree = spanningTrees[mcGroup->mtu][mcGroup->rate].spanningTree;
 		if (sm_mc_config.disable_mcast_check != McGroupBehaviorRelaxed)
 		{
@@ -917,6 +878,10 @@ void sm_add_mcmember_port_masks(void)
 			continue;
 		if (!mcGroup->mcMembers)
 			continue;
+		if (updatedVirtualFabrics && !bitset_nset(&mcGroup->new_vfMembers)) {
+			// If reconfig sweep, and new vfs is empty do not add to MFT
+			continue;
+		}
 		offset = mcGroup->mLid - STL_LID_MULTICAST_BEGIN;
 		for_all_multicast_members(mcGroup, mcMember) {
 			if (mcMember->portGuid == SA_FAKE_MULTICAST_GROUP_MEMBER) continue;
@@ -963,6 +928,13 @@ Status_t sm_calculate_mfts()
 	STL_LID	maxmcLid, lid;
 	Topology_t *topo;
 	Status_t status;
+	uint32 indexLid;
+	bitset_t allMlids;
+
+	if (!bitset_init(&sm_pool, &allMlids, (STL_LID_MULTICAST_END-STL_LID_MULTICAST_BEGIN+1))) {
+		IB_LOG_ERROR_FMT(__func__, "Out of memory.");
+		return VSTATUS_NOMEM;
+	}
 
 	topo = &sm_newTopology;
 
@@ -976,6 +948,14 @@ Status_t sm_calculate_mfts()
 	}
 
 	sm_check_mc_groups_realizable();
+	for_all_multicast_groups(mcGroup) {
+		if (updatedVirtualFabrics && !bitset_nset(&mcGroup->new_vfMembers)) {
+			// If reconfig sweep, and new vfs is empty do not add to MFT
+			continue;
+		}
+		indexLid = mcGroup->mLid - STL_LID_MULTICAST_BEGIN;
+		bitset_set(&allMlids,indexLid);
+	}
 
 	sm_calculate_spanning_tree_port_mask();
 
@@ -987,6 +967,9 @@ Status_t sm_calculate_mfts()
 
 	for_all_switch_nodes(topo, nodep) {
 		for (lid = STL_LID_MULTICAST_BEGIN; lid <= maxmcLid; lid++) {
+			offset = lid-STL_LID_MULTICAST_BEGIN;
+			if (!bitset_test(&allMlids, offset))
+				continue;
 			tree = NULL;
 			if (uniqueSpanningTreeCount == 1) {
 				tree = uniqueSpanningTrees[0];
@@ -994,6 +977,10 @@ Status_t sm_calculate_mfts()
 				for_all_multicast_groups(mcGroup) {
 					if (mcGroup->flags & McGroupUnrealizable)
 						continue;
+					if (updatedVirtualFabrics && !bitset_nset(&mcGroup->new_vfMembers)) {
+						// If reconfig sweep, and new vfs is empty do not add to MFT
+						continue;
+					}
 					if (mcGroup->mLid == lid) {
 						tree = spanningTrees[mcGroup->mtu][mcGroup->rate].spanningTree;
 						break;
@@ -1004,7 +991,7 @@ Status_t sm_calculate_mfts()
 				continue;
 			if (!tree->first_mlid)
 				continue;
-			offset = lid - STL_LID_MULTICAST_BEGIN;
+
 			if (lid != tree->first_mlid) {
 				first_mlid_offset = tree->first_mlid - STL_LID_MULTICAST_BEGIN;
 				memcpy((void *)nodep->mft[offset], (void *)nodep->mft[first_mlid_offset],
@@ -1035,7 +1022,7 @@ Status_t sm_calculate_mfts()
 
     if (sm_mc_config.enable_pruning) {
 		for (i = 0; i < uniqueSpanningTreeCount; i++) {
-			sm_pruneMcastSpanningTree(uniqueSpanningTrees[i]);
+			sm_pruneMcastSpanningTree(uniqueSpanningTrees[i], &allMlids);
 		}
 	}
 
@@ -1043,6 +1030,8 @@ Status_t sm_calculate_mfts()
 	for_all_switch_nodes(topo, nodep) {
 		for (lid = STL_LID_MULTICAST_BEGIN; lid <= maxmcLid; lid++) {
 			offset = lid - STL_LID_MULTICAST_BEGIN;
+			if (!bitset_test(&allMlids, offset))
+				continue;
 	        if (!nodep->mftPortMaskChange && topology_passcount && sm_valid_port((portp = sm_get_port(nodep, 0)))) {
 	   	        if ((oldswp = lidmap[portp->portData->lid].oldNodep)) {
 					if (memcmp((void *)nodep->mft[offset], (void *)oldswp->mft[offset], sizeof(STL_PORTMASK) * STL_NUM_MFT_POSITIONS_MASK))
@@ -1051,13 +1040,14 @@ Status_t sm_calculate_mfts()
         	}
 		}
 	}
+	bitset_free(&allMlids);
 
    (void)vs_unlock(&sm_McGroups_lock);
 
 	return VSTATUS_OK;
 }
 
-void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST)
+void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST, bitset_t *allmlids )
 {
 	int			i, j;
     int         loneEntryPos=0;
@@ -1079,6 +1069,8 @@ void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST)
      // prune out switches with only one entry
 	for (lid = STL_LID_MULTICAST_BEGIN; lid <= maxmcLid; lid++) {
 		offset = lid - STL_LID_MULTICAST_BEGIN;
+		if (!bitset_test(allmlids,offset))
+			continue;
         for (i = 0; i < mcST->num_nodes; i++) {
             mcNode = &mcST->nodes[i];
             if (!mcNode) {
@@ -1151,23 +1143,45 @@ void sm_pruneMcastSpanningTree(McSpanningTree_t *mcST)
 /*
  *  copy the old mfts to new topology
  */
-void sm_multicast_switch_mft_copy() {
+Status_t sm_multicast_switch_mft_copy() {
 	Topology_t		*newtopop;
 	Node_t			*oldswp, *newswp;
 	Port_t			*portp;
     STL_LID         offset, lid;
-	STL_LID			maxmcLid;
+	STL_LID			maxmcLid, indexLid;
+	bitset_t		allMlids;
+	Status_t		status;
+	McGroup_t		*mcGroup;
 
     newtopop = (Topology_t *)&sm_newTopology;
 
 	maxmcLid = sm_multicast_get_max_lid();
 	ASSERT(maxmcLid < sm_mcast_mlid_table_cap+STL_LID_MULTICAST_BEGIN);
 
+	if (!bitset_init(&sm_pool, &allMlids, (STL_LID_MULTICAST_END-STL_LID_MULTICAST_BEGIN+1))) {
+		IB_LOG_ERROR_FMT(__func__, "Out of memory.");
+		return VSTATUS_NOMEM;
+	}
+
+	if ((status = vs_lock(&sm_McGroups_lock)) != VSTATUS_OK) {
+		IB_LOG_ERRORRC("Failed to get sm_McGroups_lock rc:", status);
+		return status;
+	}
+
+	for_all_multicast_groups(mcGroup) {
+		indexLid = mcGroup->mLid - STL_LID_MULTICAST_BEGIN;
+		bitset_set(&allMlids,indexLid);
+	}
+
+   (void)vs_unlock(&sm_McGroups_lock);
+
     for_all_switch_nodes(newtopop, newswp) {
         if (sm_valid_port((portp = sm_get_port(newswp,0)))) {
             if ((oldswp = lidmap[portp->portData->lid].oldNodep)) {
                 for (lid = STL_LID_MULTICAST_BEGIN; lid <= maxmcLid; ++lid) {
                     offset = lid - STL_LID_MULTICAST_BEGIN;
+                    if (!bitset_test(&allMlids,offset))
+                       continue;
                     /* copy the mft over into new topology */
                     memcpy(newswp->mft[offset], oldswp->mft[offset],
                            sizeof(STL_PORTMASK) * STL_NUM_MFT_POSITIONS_MASK);
@@ -1175,6 +1189,8 @@ void sm_multicast_switch_mft_copy() {
             }
         }
     }
+	bitset_free(&allMlids);
+	return VSTATUS_OK;
 }
 
 // NOTE: Assumes both curr_tp and prev_tp are properly locked.
@@ -1196,7 +1212,15 @@ Status_t sm_set_all_mft(int force, Topology_t *curr_tp, Topology_t *prev_tp)
 	SmpAddr_t		addr;
 	uint64_t		sTime, eTime;
 	int				dispatched = 0, mftBlockChange = 0;
-	uint16_t		old_portMask = 0;
+	STL_PORTMASK	old_portMask = 0;
+	bitset_t		allMlids;
+	uint32_t		indexLid, offset;
+	McGroup_t		*mcGroup;
+
+	if (!bitset_init(&sm_pool, &allMlids, (STL_LID_MULTICAST_END-STL_LID_MULTICAST_BEGIN+1))) {
+		IB_LOG_ERROR_FMT(__func__, "Out of memory.");
+		return VSTATUS_NOMEM;
+	}
 
 	if (smDebugPerf) {
 		vs_time_get(&sTime);
@@ -1205,6 +1229,21 @@ Status_t sm_set_all_mft(int force, Topology_t *curr_tp, Topology_t *prev_tp)
 			IB_LOG_INFINI_INFO0("Forcing complete MFT reprogramming");
 	}
 
+	if ((status = vs_lock(&sm_McGroups_lock)) != VSTATUS_OK) {
+		IB_LOG_ERRORRC("Failed to get sm_McGroups_lock rc:", status);
+		return status;
+	}
+
+	for_all_multicast_groups(mcGroup) {
+		if (updatedVirtualFabrics && !bitset_nset(&mcGroup->new_vfMembers)) {
+			// If reconfig sweep, and new vfs is empty do not add to MFT
+			continue;
+		}
+		indexLid = mcGroup->mLid - STL_LID_MULTICAST_BEGIN;
+		bitset_set(&allMlids,indexLid);
+	}
+	vs_unlock(&sm_McGroups_lock);
+
 	newMaxLid = sm_multicast_get_max_lid();
 
 	if (oldMaxLid > newMaxLid) {
@@ -1243,18 +1282,20 @@ Status_t sm_set_all_mft(int force, Topology_t *curr_tp, Topology_t *prev_tp)
 			for (i = 0; i < STL_NUM_MFT_POSITIONS_MASK && i * STL_PORT_MASK_WIDTH <= switchp->nodeInfo.NumPorts; ++i) {
 				mftBlockChange = 0;
 				for (j = 0; j < STL_NUM_MFT_ELEMENTS_BLOCK; j++) {
-					uint16_t offset = (lid - STL_LID_MULTICAST_BEGIN);
+					offset = (lid - STL_LID_MULTICAST_BEGIN);
 					if (offset+j < sm_mcast_mlid_table_cap)
 						mft.MftBlock[j] = switchp->mft[offset + j][i];
 					else
 						mft.MftBlock[j] = 0;
-
 					if (mftBlockChange)
 						continue;
 
 					if ((lid + j) > maxLid)
 						break;
 
+					if (!bitset_test(&allMlids, offset + j))
+						continue;
+
 					if ((lid + j) > oldMaxLid) {
 						/* new multicast lid */
 						if (mft.MftBlock[j] != 0)
@@ -1287,6 +1328,7 @@ Status_t sm_set_all_mft(int force, Topology_t *curr_tp, Topology_t *prev_tp)
 			}
 		}
 	}
+	bitset_free(&allMlids);
 
 	if (dispatched) {
 		status = sm_dispatch_wait(&sm_asyncDispatch);
@@ -1595,9 +1637,8 @@ sm_multicast_get_mlid(void)
 	if (ListCount(&free_mlid_list)) {
 		mlid = (STL_LID)(uintn)ListRemoveHead(&free_mlid_list);
 	} else if (highwater_mlid < (STL_LID_MULTICAST_BEGIN+sm_mcast_mlid_table_cap)) {
-		mlid = highwater_mlid++;
+			mlid = highwater_mlid++;
 	}
-
 	vs_unlock(&free_mlid_lock);
 	return mlid;
 }
@@ -1680,6 +1721,8 @@ sm_multicast_get_max_lid() {
 void
 sm_multicast_mark_lid_in_use(STL_LID mlid) {
 	ASSERT(mlid < sm_mcast_mlid_table_cap+STL_LID_MULTICAST_BEGIN);
+
+
 	if (mlid >= highwater_mlid) {
 		highwater_mlid = mlid+1;
 	} else {
@@ -1691,7 +1734,7 @@ sm_multicast_mark_lid_in_use(STL_LID mlid) {
 		for (i=ListHead(&free_mlid_list);
 			i != NULL && (STL_LID)(uintn)ListObj(i) < mlid;
 			i=ListNext(&free_mlid_list, i));
-	
+
 		// Remove this mlid from the free list.
 		if (i != NULL && (STL_LID)(uintn)ListObj(i) == mlid) {
 			ListRemoveItem(&free_mlid_list,i);
@@ -1808,7 +1851,7 @@ sm_mc_get_group_class_lid(McGroupClass_t * groupClass, PKey_t pKey, uint8_t mtu,
 		}
 	}
 
-	if (groupClass->currentLids < groupClass->maximumLids)
+	if (groupClass->maximumLids == 0 || groupClass->currentLids < groupClass->maximumLids)
 	{
 		if (lidClass == NULL)
 		{
@@ -2170,7 +2213,7 @@ sm_multicast_sync_lid(IB_GID mGid, PKey_t pKey, uint8_t mtu, uint8_t rate, STL_L
  */
 Status_t
 sm_multicast_assign_lid(IB_GID mGid, PKey_t pKey, uint8_t mtu, uint8_t rate,
-                        STL_LID * lid)
+                        STL_LID requestedLid, STL_LID * lid)
 {
 	Status_t status = VSTATUS_OK;
 	McGroupClass_t * groupClass = NULL;
@@ -2187,7 +2230,9 @@ sm_multicast_assign_lid(IB_GID mGid, PKey_t pKey, uint8_t mtu, uint8_t rate,
 	}
 #endif
 
-	if (groupClass->maximumLids == 0)
+	if (requestedLid != 0) {
+		status = sm_mc_get_group_class_lid(groupClass, pKey, mtu, rate, requestedLid, lid);
+	} else if (groupClass->maximumLids == 0)
 	{
 		/* Non-shared MLIDs for this MGID - just generate a new mcast
 		 * lid and return */
