@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT7 ****************************************
 
-Copyright (c) 2015-2017, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -342,7 +342,8 @@ sm_routing_route_old_switch(Topology_t *src_topop, Topology_t *dst_topop, Node_t
 
 	if ((new_endnodesInUse.nset_m ||
 		 src_topop->num_endports != dst_topop->num_endports) &&
-		dst_topop->routingModule->funcs.can_send_partial_routes()) {
+		dst_topop->routingModule->funcs.can_send_partial_routes() &&
+		!dst_topop->routingModule->funcs.needs_routing_recalc(dst_topop, nodep)) {
 		if (sm_config.sm_debug_routing)
 			IB_LOG_INFINI_INFO_FMT(__func__, "Partial LFT send - switch %s", sm_nodeDescString(nodep));
 		sm_send_partial_lft(sm_topop, nodep, &sm_topop->deltaLidBlocks);
@@ -620,8 +621,19 @@ sm_routing_init(void)
 	return status;
 }
 
+uint8_t
+sm_get_slsc(Topology_t *topop, Port_t* portp, uint8_t sl)
+{
+	// caller check not switch
+	if (sl >= STL_MAX_SLS)
+		return 0;
+
+	return (portp->portData->slscMap.SLSCMap[sl].SC);
+}
+
 int
-sm_get_route(Topology_t *topop, Node_t *switchp, uint8_t inport, STL_LID dlid) {
+sm_get_route(Topology_t *topop, Node_t *switchp, uint8_t inport, STL_LID dlid, uint8_t* sc)
+{
 
 
 	return switchp->lft ? switchp->lft[dlid] : 0xff;
