@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT7 ****************************************
 
-Copyright (c) 2015-2018, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -459,6 +459,60 @@ FSTATUS FindNodeNamePoint(FabricData_t *fabricp, char *name, Point *pPoint, uint
 }
 
 #ifndef __VXWORKS__
+
+/* append searched objects that match the pattern */
+FSTATUS PopoulateNodePatPairs(NodePairList_t *nodePatPairs, uint8 side, void *object)
+{
+	DLIST *pList;
+
+	if (side == LSIDE_PAIR){
+		pList = &nodePatPairs->nodePairList1;
+	} else if(side == RSIDE_PAIR) {
+		pList = &nodePatPairs->nodePairList2;
+	} else {
+		return FINVALID_OPERATION;
+	}
+
+	if (!ListInsertTail(pList, object)) {
+		fprintf(stderr, "%s: unable to allocate memory\n", g_Top_cmdname);
+		return FINSUFFICIENT_MEMORY;
+	}
+	return FSUCCESS;
+}
+
+// search for the NodeData
+// corresponding to the given node name pattern for the given Node Pair
+// FNOT_FOUND - no instances found
+// FINVALID_OPERATION - find_flag or side contains no applicable searches
+// other - error allocating memory or initializing structures
+FSTATUS FindNodePatPairs(FabricData_t *fabricp, char *pattern, NodePairList_t *nodePatPairs,
+								uint8 find_flag, uint8 side)
+{
+	FSTATUS status;
+
+	if (0 == find_flag)
+		return FINVALID_OPERATION;
+
+	if (0 == side)
+		return FINVALID_OPERATION;
+
+	if (find_flag & FIND_FLAG_FABRIC){
+		cl_map_item_t *p;
+		/* the node can be of  type FI or SW */
+		for (p = cl_qmap_head(&fabricp->AllNodes); p != cl_qmap_end(&fabricp->AllNodes); p = cl_qmap_next(p)){
+			NodeData *nodep = PARENT_STRUCT(p, NodeData, AllNodesEntry);
+			/* find all SWs and FIs that match the pattern */
+			if (fnmatch(pattern, (char*)nodep->NodeDesc.NodeString, 0) == 0){
+				status = PopoulateNodePatPairs(nodePatPairs, side, nodep);
+				if (FSUCCESS != status)
+					return status;
+			}
+		}
+		return FSUCCESS;
+	}
+	return FNOT_FOUND;
+}
+
 // search for the NodeData, ExpectedNode and ExpectedSM
 // corresponding to the given node name pattern
 // FNOT_FOUND - no instances found
@@ -467,23 +521,41 @@ FSTATUS FindNodeNamePoint(FabricData_t *fabricp, char *name, Point *pPoint, uint
 FSTATUS FindNodeNamePatPoint(FabricData_t *fabricp, char *pattern, Point *pPoint, uint8 find_flag)
 {
 	FSTATUS status;
-	char Name[STL_NODE_DESCRIPTION_ARRAY_SIZE+1];
 
-	ASSERT(! PointValid(pPoint));
+	ASSERT(pPoint);
+	status = FindNodeNamePatPointUncompress(fabricp, pattern, pPoint, find_flag);
+	if (FSUCCESS == status)
+		PointCompress(pPoint);
+
+	return status;
+}
+
+// search for the NodeData, ExpectedNode and ExpectedSM
+// corresponding to the given node name pattern
+// FNOT_FOUND - no instances found
+// FINVALID_OPERATION - find_flag contains no applicable searches
+// other - error allocating memory or initializing structures
+FSTATUS FindNodeNamePatPointUncompress(FabricData_t *fabricp, char *pattern, Point *pPoint, uint8 find_flag)
+{
+	FSTATUS status;
+
+	ASSERT(pPoint);
 	if (0 == find_flag)
 		return FINVALID_OPERATION;
 	if (find_flag & FIND_FLAG_FABRIC) {
 		cl_map_item_t *p;
 		for (p=cl_qmap_head(&fabricp->AllNodes); p != cl_qmap_end(&fabricp->AllNodes); p = cl_qmap_next(p)) {
 			NodeData *nodep = PARENT_STRUCT(p, NodeData, AllNodesEntry);
-
-			strncpy(Name, (char*)nodep->NodeDesc.NodeString, STL_NODE_DESCRIPTION_ARRAY_SIZE);
-			Name[STL_NODE_DESCRIPTION_ARRAY_SIZE] = '\0';
-			if (fnmatch(pattern, Name, 0) == 0)
+			if (fnmatch(pattern, (char*)nodep->NodeDesc.NodeString, 0) == 0)
 			{
 				status = PointListAppend(pPoint, POINT_TYPE_NODE_LIST, nodep);
 				if (FSUCCESS != status)
 					return status;
+				//Set flag if the node is a switch or FI
+				if (nodep->NodeInfo.NodeType == STL_NODE_SW)
+					pPoint->haveSW = TRUE;
+				else if (nodep->NodeInfo.NodeType == STL_NODE_FI)
+					pPoint->haveFI = TRUE;
 			}
 		}
 	}
@@ -540,7 +612,6 @@ FSTATUS FindNodeNamePatPoint(FabricData_t *fabricp, char *pattern, Point *pPoint
 						g_Top_cmdname, pattern);
 		return FNOT_FOUND;
 	}
-	PointCompress(pPoint);
 	return FSUCCESS;
 }
 
@@ -2038,7 +2109,7 @@ ExpectedLink* FindExpectedLinkByOneSide(const FabricData_t *fabricp, EUI64 nodeG
 // FNOT_FOUND - no instances found
 // FINVALID_OPERATION - find_flag contains no applicable searches
 // other - error allocating memory or initializing structures
-FSTATUS FindLinkQualityPoint(FabricData_t *fabricp, uint16 quality, LinkQualityCompare comp, Point *pPoint, uint8 find_flag)
+FSTATUS FindLinkCRCPoint(FabricData_t *fabricp, uint16 crc, LinkCRCCompare comp, Point *pPoint, uint8 find_flag)
 {
 	FSTATUS status;
 
@@ -2051,17 +2122,15 @@ FSTATUS FindLinkQualityPoint(FabricData_t *fabricp, uint16 quality, LinkQualityC
 			PortData *portp = (PortData *)QListObj(p);
 
 			boolean match = FALSE;
-			if (!portp->pPortStatus)
+			/* omit switch port 0, CRC is often odd and N/A */
+			if (portp->PortNum == 0)
 				continue;
 			switch (comp) {
-			case QUAL_EQ:
-				match = (uint16)(portp->pPortStatus->lq.s.LinkQualityIndicator) == quality;
+			case CRC_EQ:
+				match = (portp->PortInfo.PortLTPCRCMode.s.Active == crc );
 				break;
-			case QUAL_GE:
-				match = (uint16)(portp->pPortStatus->lq.s.LinkQualityIndicator) >= quality;
-				break;
-			case QUAL_LE:
-				match = (uint16)(portp->pPortStatus->lq.s.LinkQualityIndicator) <= quality;
+			case CRC_NE:
+				match = (portp->PortInfo.PortLTPCRCMode.s.Active != crc );
 				break;
 			default:
 				break;
@@ -2077,14 +2146,126 @@ FSTATUS FindLinkQualityPoint(FabricData_t *fabricp, uint16 quality, LinkQualityC
 	// N/A for FIND_FLAG_ENODE, FIND_FLAG_ESM and FIND_FLAG_ELINK
 
 	if (! PointValid(pPoint)) {
-		fprintf(stderr, "%s: Link Quality Not Found: %d\n",
-						g_Top_cmdname, quality);
+		char tempbuf[20];
+		fprintf(stderr, "%s: Link CRC Not Found: %s %s\n",
+						g_Top_cmdname,
+						(comp == CRC_EQ)? "EQ"
+						: (comp == CRC_NE) ? "NE"
+						: "", /* should not happen */
+						StlPortLtpCrcModeToText(crc, tempbuf, sizeof(tempbuf)));
 		return FNOT_FOUND;
 	}
 	PointCompress(pPoint);
 	return FSUCCESS;
 }
 
+// FNOT_FOUND - no instances found
+// FINVALID_OPERATION - find_flag contains no applicable searches
+// other - error allocating memory or initializing structures
+FSTATUS FindLinkQualityPoint(FabricData_t *fabricp, uint16 quality, LinkQualityCompare comp, Point *pPoint, uint8 find_flag)
+{
+	FSTATUS status;
+
+	ASSERT(! PointValid(pPoint));
+	if (0 == (find_flag & FIND_FLAG_FABRIC))
+		return FINVALID_OPERATION;
+	if (find_flag & FIND_FLAG_FABRIC) {
+		LIST_ITEM *p;
+		for (p=QListHead(&fabricp->AllPorts); p != NULL; p = QListNext(&fabricp->AllPorts, p)) {
+			PortData *portp = (PortData *)QListObj(p);
+
+			boolean match = FALSE;
+			if (!portp->pPortCounters)
+				continue;
+			switch (comp) {
+			case QUAL_EQ:
+				match = (uint16)(portp->pPortCounters->lq.s.linkQualityIndicator) == quality;
+				break;
+			case QUAL_GE:
+				match = (uint16)(portp->pPortCounters->lq.s.linkQualityIndicator) >= quality;
+				break;
+			case QUAL_LE:
+				match = (uint16)(portp->pPortCounters->lq.s.linkQualityIndicator) <= quality;
+				break;
+			default:
+				break;
+			}
+			if (match) {
+				status = PointListAppend(pPoint, POINT_TYPE_PORT_LIST, portp);
+				if (FSUCCESS != status) 
+					return status;
+			}
+		}
+	}
+
+	// N/A for FIND_FLAG_ENODE, FIND_FLAG_ESM and FIND_FLAG_ELINK
+
+	if (! PointValid(pPoint)) {
+		fprintf(stderr, "%s: Link Quality Not Found: %s %d\n",
+						g_Top_cmdname,
+						(comp == QUAL_EQ)? "EQ"
+						: (comp == QUAL_GE) ? "GE"
+						: (comp == QUAL_LE) ? "LE"
+						: "", /* should not happen */
+						quality);
+		return FNOT_FOUND;
+	}
+	PointCompress(pPoint);
+	return FSUCCESS;
+}
+
+FSTATUS FindLinkDownReasonPoint(FabricData_t *fabricp, uint8 ldr, Point *pPoint, uint8 find_flag)
+{
+	FSTATUS status;
+
+	ASSERT(PointIsInInit(pPoint));
+	if (0 == (find_flag & FIND_FLAG_FABRIC))
+		return FINVALID_OPERATION;
+
+	if (find_flag & FIND_FLAG_FABRIC) {
+		LIST_ITEM *p;
+		for (p=QListHead(&fabricp->AllPorts); p != NULL; p = QListNext(&fabricp->AllPorts, p)) {
+			PortData *portp = (PortData *)QListObj(p);
+
+			boolean match = FALSE;
+			int i;
+			if (fabricp->flags & FF_SMADIRECT) { // SMA only
+				match = ldr == IB_UINT8_MAX
+					? portp->PortInfo.LinkDownReason != STL_LINKDOWN_REASON_NONE
+					: portp->PortInfo.LinkDownReason == ldr;
+				match |= ldr == IB_UINT8_MAX
+					? portp->PortInfo.NeighborLinkDownReason != STL_LINKDOWN_REASON_NONE
+					: portp->PortInfo.NeighborLinkDownReason == ldr;
+			} else { // SA
+				for (i = 0; i < STL_NUM_LINKDOWN_REASONS; ++i) {
+					STL_LINKDOWN_REASON *ldrp = &portp->LinkDownReasons[i];
+					if (ldrp->Timestamp != 0) {
+						match = ldr == IB_UINT8_MAX
+							? ldrp->LinkDownReason != STL_LINKDOWN_REASON_NONE
+							: ldrp->LinkDownReason == ldr;
+						match |= ldr == IB_UINT8_MAX
+							? ldrp->NeighborLinkDownReason != STL_LINKDOWN_REASON_NONE
+							: ldrp->NeighborLinkDownReason == ldr;
+					}
+					if (match) break;
+				}
+			}
+			if (match) {
+				status = PointListAppend(pPoint, POINT_TYPE_PORT_LIST, portp);
+				if (FSUCCESS != status) 
+					return status;
+			}
+		}
+	}
+	if (! PointValid(pPoint)) {
+		fprintf(stderr, "%s: Link Down Reason Not Found: %d\n", g_Top_cmdname, ldr);
+		return FNOT_FOUND;
+	}
+	PointCompress(pPoint);
+	return FSUCCESS;
+}
+
+
 // Search through the ExpectedSMs for matching portGuid
 // FNOT_FOUND - no instances found
 // FINVALID_PARAMETER - input parameter not valid
