@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT7 ****************************************
 
-Copyright (c) 2015-2017, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -49,6 +49,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <stdlib.h>
 #include <stdarg.h>
 #include <unistd.h>
+#include <sys/stat.h>
+
 #if !defined(_GNU_SOURCE)
 #define _GNU_SOURCE
 #endif
@@ -97,8 +99,6 @@ typedef struct QOSData_s {
 } QOSData;
 
 
-typedef STL_PORT_STATUS_RSP STL_PortStatusData_t;
-
 // How many STL_CABLE_INFO structs to store per port
 #define PORTDATA_CABLEINFO_SIZE	4
 #define IFACE_MACLIST_SIZE		512
@@ -122,11 +122,12 @@ typedef struct PortData_s {
 	uint32 rate;			// Active rate for this port
 	STL_LID	EndPortLID;				// LID to get to device with this port
 	STL_PORT_INFO	PortInfo;			// do not use LocalPortNum,use PortNum above
+	STL_LINKDOWN_REASON LinkDownReasons[STL_NUM_LINKDOWN_REASONS];
 
 
 	STL_LED_INFO LedInfo;			//Led Info for this port
 	IB_PATH_RECORD *pathp;			// Path Record to send to this port
-	STL_PortStatusData_t *pPortStatus;
+	STL_PORT_COUNTERS_DATA *pPortCounters;
 	struct ExpectedLink_s *elinkp;	// if supplied in topology input
 	QOSData		*pQOS;				// optional QOS
 	STL_PKEY_ELEMENT	*pPartitionTable;	// optional Partition Table
@@ -667,6 +668,19 @@ typedef struct Top_FreeCallbacks_s {
 	FabricDataFreeCallback	*pFabricDataFreeCallback;
 } Top_FreeCallbacks;
 
+// For functions which generate Points, is it node pair or just node
+#define PAIR_FLAG_NONE		0x01	/* no pair exists */
+#define PAIR_FLAG_NODE		0x02	/* pair exists */
+
+// Identifies side of pair
+#define LSIDE_PAIR		0x01	/* left side of pair */
+#define RSIDE_PAIR		0x02	/* right side of pair */
+
+typedef struct NodePairList_s {
+	DLIST			nodePairList1;     //members of left side of pair
+	DLIST			nodePairList2;     //members of right side of pair
+} NodePairList_t;
+
 /* struct Point_s identifies a particular point in the fabric and
  * topology.xml.
  * Used for trace route and other "focused" reports
@@ -684,6 +698,7 @@ typedef enum {
 	POINT_TYPE_IOC_LIST,
 #endif
 	POINT_TYPE_SYSTEM,
+	POINT_TYPE_NODE_PAIR_LIST,
 } PointType;
 
 typedef enum {
@@ -709,6 +724,8 @@ typedef struct Point_s {
 
 	/* object(s) matched in the fabric */
 	PointType	Type;	/* if POINT_TYPE_NONE, u undefined */
+	boolean		haveSW;	/* indicates the point has SW*/
+	boolean		haveFI;	/* indicates the point has FI*/
 	union {
 		PortData	*portp;
 		NodeData	*nodep;
@@ -719,6 +736,7 @@ typedef struct Point_s {
 		DLIST		nodeList;
 		DLIST		portList;
 		DLIST		iocList;
+		NodePairList_t		nodePairList;
 	} u;
 
 	/* ExpectedNode(s) matched in topology file */
@@ -773,6 +791,11 @@ typedef enum {
 	QUAL_LE
 } LinkQualityCompare;
 
+typedef enum {
+	CRC_EQ,
+	CRC_NE
+} LinkCRCCompare;
+
 // simple way to convert time_t to a localtime date string in dest
 // (from Topology/getdate.c)
 extern void Top_formattime(char *dest, size_t max, time_t t);
@@ -974,6 +997,9 @@ extern NodeData * FindNodeGuid(const FabricData_t* fabricp, EUI64 guid);
 extern FSTATUS FindNodeGuidPoint(FabricData_t *fabricp, EUI64 guid, Point *pPoint, uint8 find_flag, int silent);
 extern FSTATUS FindNodeNamePoint(FabricData_t* fabricp, char *name, Point *pPoint, uint8 find_flag, int silent);
 extern FSTATUS FindNodeNamePatPoint(FabricData_t* fabricp, char *pattern, Point *pPoint, uint8 find_flag);
+extern FSTATUS FindNodeNamePatPointUncompress(FabricData_t *fabricp, char *pattern, Point *pPoint, uint8 find_flag);
+
+extern FSTATUS FindNodePatPairs(FabricData_t *fabricp, char *pattern, NodePairList_t *nodePatPairs, uint8 find_flag, uint8 side);
 extern FSTATUS FindNodeDetailsPatPoint(FabricData_t* fabricp, const char* pattern, Point *pPoint, uint8 find_flag);
 extern FSTATUS FindNodeTypePoint(FabricData_t* fabricp, NODE_TYPE type, Point *pPoint, uint8 find_flag);
 #if !defined(VXWORKS) || defined(BUILD_DMC)
@@ -1011,7 +1037,9 @@ extern PortData * FindNodeGuidPort(FabricData_t *fabricp,EUI64 nodeguid, uint8 p
 extern ExpectedNode* FindExpectedNodeByNodeGuid(const FabricData_t* fabricp, EUI64 nodeGuid);
 extern ExpectedNode* FindExpectedNodeByNodeDesc(const FabricData_t* fabricp, const char* nodeDesc, uint8 NodeType);
 extern ExpectedLink* FindExpectedLinkByOneSide(const FabricData_t* fabricp, EUI64 nodeGuid, uint8 portNum, uint8* side);
+extern FSTATUS FindLinkCRCPoint(FabricData_t *fabricp, uint16 crc, LinkCRCCompare comp, Point *pPoint, uint8 find_flag);
 extern FSTATUS FindLinkQualityPoint(FabricData_t *fabricp, uint16 quality, LinkQualityCompare comp, Point *pPoint, uint8 find_flag);
+extern FSTATUS FindLinkDownReasonPoint(FabricData_t *fabricp, uint8 ldr, Point *pPoint, uint8 find_flag);
 extern FSTATUS FindExpectedSMByPortGuid(FabricData_t *fabricp, EUI64 portGuid);
 extern FSTATUS FindExpectedSMByNodeGuid(FabricData_t *fabricp, EUI64 nodeGuid);
 
@@ -1059,7 +1087,7 @@ extern boolean NodeHasPma(NodeData *nodep);
 extern boolean PortHasPma(PortData *portp);
 extern void UpdateNodePmaCapabilities(NodeData *nodep, boolean ProcessHFICounters);
 extern FSTATUS STLPmGetClassPortInfo(struct omgt_port *port, PortData *portp);
-extern FSTATUS STLPmGetPortStatus(struct omgt_port *port, PortData *portp, uint8 portNum, STL_PortStatusData_t *pPortStatus);
+extern FSTATUS STLPmGetPortStatus(struct omgt_port *port, PortData *portp, uint8 portNum, STL_PORT_STATUS_RSP *pPortStatus);
 extern FSTATUS STLPmClearPortCounters(struct omgt_port *port, PortData *portp, uint8 lastPortIndex, uint32 counterselect);
 #if !defined(VXWORKS) || defined(BUILD_DMC)
 extern FSTATUS DmGetIouInfo(struct omgt_port *port, IB_PATH_RECORD *pathp, IOUnitInfo *pIouInfo);
@@ -1071,6 +1099,7 @@ extern FSTATUS DmGetServiceEntries(struct omgt_port *port, IB_PATH_RECORD *pathp
 
 // POINT routines (from Topology/point.c)
 extern void PointInit(Point *point);
+extern boolean PointIsInInit(Point *point);
 extern void PointFabricDestroy(Point *point);
 extern void PointEnodeDestroy(Point *point);
 extern void PointEsmDestroy(Point *point);
@@ -1086,6 +1115,8 @@ extern FSTATUS PointListAppend(Point *point, PointType type, void *object);
 extern FSTATUS PointEnodeListAppend(Point *point, PointEnodeType type, void *object);
 extern FSTATUS PointEsmListAppend(Point *point, PointEsmType type, void *object);
 extern FSTATUS PointElinkListAppend(Point *point, PointElinkType type, void *object);
+extern FSTATUS PointNodePairListAppend(Point *point, uint8 side, void *object);
+extern FSTATUS PointPopulateNodePairList(Point *pPoint, NodePairList_t *nodePatPairs);
 extern void PointFabricCompress(Point *point);
 extern void PointEnodeCompress(Point *point);
 extern void PointEsmCompress(Point *point);
@@ -1143,6 +1174,15 @@ extern boolean CompareSystemPoint(SystemData *systemp, Point *point);
 extern char* ComparePrefix(char *arg, const char *prefix);
 extern FSTATUS ParsePoint(FabricData_t *fabricp, char* arg, Point* pPoint, uint8 find_flag, char **pp);
 
+/* check if point is of Type NodePairList */
+extern boolean PointTypeIsNodePairList(Point *pPoint);
+/* check if point is of Type NodeList */
+extern boolean PointIsTypeNodeList(Point *pPoint);
+/* check if haveSW flag is set */
+extern boolean PointHaveSw(Point *pPoint);
+/* check if haveFI flag is set */
+extern boolean PointHaveFI(Point *pPoint);
+
 // snapshot input/output routines (from Topology/snapshot.c)
 extern void Xml2PrintSnapshot(FILE *file, SnapshotOutputInfo_t *info);
 
@@ -1315,7 +1355,7 @@ extern FSTATUS TabulateRoutes(FabricData_t *fabricp,
 			   		PortData *portp1, PortData *portp2, uint32 *totalPaths,
 					uint32 *badPaths, boolean fatTree);
 // tabulate all the routes between FIs, exclude loopback routes
-extern FSTATUS TabulateCARoutes(FabricData_t *fabricp, uint32 *totalPaths,
+extern FSTATUS TabulateCARoutes(FabricData_t *fabricp, Point *focus, uint32 *totalPaths,
 					uint32 *badPaths, boolean fatTree);
 
 typedef void (*ReportCallback_t)(PortData *portp1, PortData *portp2,
@@ -1532,6 +1572,78 @@ static inline SC2VLUpdateType getSC2VLUpdateType(NodeData *nodep, PortData *port
 	return SC2VL_UPDATE_TYPE_NONE;
 }
 
+/* Port Iterator structure to hold information of the port being iterated */
+typedef struct _PortIteratorData
+{
+	boolean lastPortFlag;
+}PortIteratorData;
+
+/* Port List Iterator structure to hold information of the port List being iterated */
+typedef struct _PortListIteratorData
+{
+	LIST_ITERATOR currentPort;
+}PortListIteratorData;
+
+/* Node Iterator structure to hold information of the node being iterated */
+typedef struct _NodeIteratorData
+{
+	boolean lastNodeFlag;
+	cl_map_item_t *pCurrentPort;
+}NodeIteratorData;
+
+/* Node List Iterator structure to hold information of the node list being iterated */
+typedef struct _NodeListIteratorData
+{
+	LIST_ITERATOR currentNode;
+	cl_map_item_t *pCurrentPort;
+}NodeListIteratorData;
+
+/* Ioc Iterator structure to hold information of the Ioc being iterated */
+#if !defined(VXWORKS) || defined(BUILD_DMC)
+typedef struct _IocIteratorData
+{
+	boolean lastIocFlag;
+	cl_map_item_t *pCurrentPort;
+}IocIteratorData;
+
+/* Ioc List Iterator structure to hold information of the Ioc Lst being iterated */
+typedef struct _IocListIteratorData
+{
+	LIST_ITERATOR currentNode;
+	cl_map_item_t *pCurrentPort;
+}IocListIteratorData;
+#endif
+
+/* System Iterator structure to hold information of the system being iterated */
+typedef struct _SystemListIteratorData
+{
+	boolean lastSystemFlag;
+	cl_map_item_t *pCurrentNode;
+	cl_map_item_t *pCurrentPort;
+}SystemListIteratorData;
+
+/* Iterator structure to hold information of the point being iterated */
+typedef struct _FIPortIterator
+{
+	Point *pPoint;
+	union {
+		PortIteratorData		PortIter;
+		NodeIteratorData		NodeIter;
+#if !defined(VXWORKS) || defined(BUILD_DMC)
+		IocIteratorData			IocIter;
+		IocListIteratorData 	IocListIter;
+#endif
+		SystemListIteratorData	SystemIter;
+		NodeListIteratorData	NodeListIter;
+		PortListIteratorData	PortListIter;
+	} u;
+}FIPortIterator;
+
+/*Finds the next non-SW port for each type in the point*/
+extern PortData *FIPortIteratorHead(FIPortIterator *pFIPortIterator, Point *pFocus);
+/* Finds the first non-SW port for each type in the point.*/
+extern PortData *FIPortIteratorNext(FIPortIterator *pFIPortIterator);
+
 #ifdef __cplusplus
 };
 #endif
