@@ -1,6 +1,6 @@
 /* BEGIN_ICS_COPYRIGHT2 ****************************************
 
-Copyright (c) 2015-2017, Intel Corporation
+Copyright (c) 2015-2020, Intel Corporation
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -341,12 +341,15 @@ extern void IXmlOutputHex64(IXmlOutputState_t *state, const char *tag, uint64 va
 extern void IXmlOutputOptionalHex64(IXmlOutputState_t *state, const char *tag, uint64 value);
 extern void IXmlOutputPrintStrLen(IXmlOutputState_t *state, const char* value, int len);
 extern void IXmlOutputPrintStr(IXmlOutputState_t *state, const char* value);
+extern void IXmlOutputPrintStrNewlineContent(IXmlOutputState_t *state, const char* value);
 extern void IXmlOutputStrLen(IXmlOutputState_t *state, const char *tag, const char* value, int len);
 extern void IXmlOutputOptionalStrLen(IXmlOutputState_t *state, const char *tag, const char* value, int len);
 extern void IXmlOutputStr(IXmlOutputState_t *state, const char *tag, const char* value);
 extern void IXmlOutputOptionalStr(IXmlOutputState_t *state, const char *tag, const char* value);
 extern void IXmlOutputStrUint(IXmlOutputState_t *state, const char *tag, const char* str, unsigned value);
 extern void IXmlOutputOptionalStrUint(IXmlOutputState_t *state, const char *tag, const char* str, unsigned value);
+extern void IXmlOutputStrUint64(IXmlOutputState_t *state, const char *tag, const char* str, uint64 value);
+extern void IXmlOutputOptionalStrUint64(IXmlOutputState_t *state, const char *tag, const char* str, uint64 value);
 extern void IXmlOutputStrInt(IXmlOutputState_t *state, const char *tag, const char* str, int value);
 extern void IXmlOutputOptionalStrInt(IXmlOutputState_t *state, const char *tag, const char* str, int value);
 extern void IXmlOutputStruct(IXmlOutputState_t *state, const char *tag, void *data,
@@ -492,6 +495,11 @@ extern void IXmlParserEndNoop(struct IXmlParserState *state,
 /* return TRUE if current field's contents are empty or all whitespace */
 boolean IXmlIsWhitespace(const XML_Char *str, boolean *hasNewline);
 
+/* discard trailing whitespace in str in last line, return new length
+ * str modified in place
+ */
+unsigned IXmlTrimTrailingSpaces(XML_Char *str, unsigned len);
+
 /* discard leading and trailing whitespace in str, return new length
  * str modified in place
  */
